<?php

require_once __DIR__ . '/vendor/autoload.php';

use setasign\Fpdi\Tcpdf\Fpdi;
use Endroid\QrCode\QrCode;
use Endroid\QrCode\Writer\PngWriter;
use Endroid\QrCode\Color\Color;
use Endroid\QrCode\Encoding\Encoding;
use Endroid\QrCode\ErrorCorrectionLevel;
use Endroid\QrCode\RoundBlockSizeMode;

// Configuración del certificado
$certificadoP12 = __DIR__ . '/certificados/LUISALBERTO.p12';
$passwordP12 = 'Luis1982';
$publicCertPath = __DIR__ . '/certificados/public_cert.pem';
$privateKeyPath = __DIR__ . '/certificados/private_key.pem';
$rutaQR = __DIR__ . '/qr_firma.png'; // Ruta para guardar el archivo QR

// Verificación del archivo P12
if (!file_exists($certificadoP12)) {
    die("Error: Archivo P12 no encontrado.");
}

// Leer archivo P12
try {
    $pfxContent = file_get_contents($certificadoP12);
    if (!openssl_pkcs12_read($pfxContent, $certDetails, $passwordP12)) {
        throw new Exception("Error al leer el archivo P12. Verifica la contraseña y el archivo.");
    }
} catch (Exception $e) {
    die($e->getMessage());
}

// Extraer certificado y clave privada
$publicCert = $certDetails['cert'];
$privateKey = $certDetails['pkey'];
file_put_contents($publicCertPath, $publicCert);
file_put_contents($privateKeyPath, $privateKey);

// Validación del certificado
$certInfo = openssl_x509_parse($publicCert);
$actualTime = time();

if ($actualTime >= $certInfo['validFrom_time_t'] && $actualTime <= $certInfo['validTo_time_t']) {
    echo "El certificado es válido.\n";
} else {
    die("Error: El certificado ha expirado o aún no es válido.");
}

// Generar código QR con fondo transparente
function generarCodigoQR($contenido, $rutaQR)
{
    // Crear el QR normal
    $writer = new PngWriter();
    $qrCode = new QrCode(
        data: $contenido,
        encoding: new Encoding('UTF-8'),
        errorCorrectionLevel: ErrorCorrectionLevel::Low,
        size: 55, // Tamaño del QR
        margin: 0, // Sin margen
        roundBlockSizeMode: RoundBlockSizeMode::Margin,
        foregroundColor: new Color(0, 0, 0), // Color de los bloques del QR (negro)
        backgroundColor: new Color(255, 255, 255) // Fondo blanco temporal
    );
    $result = $writer->write($qrCode);
    $result->saveToFile($rutaQR);

    // Convertir el fondo blanco en transparente usando Imagick
    $imagick = new \Imagick($rutaQR);
    $imagick->setImageFormat('png');
    $imagick->transparentPaintImage(
        new ImagickPixel("white"), // Cambiar el blanco a transparente
        0.0, // Tolerancia
        0, // Antialias
        false
    );
    $imagick->writeImage($rutaQR); // Guardar el QR con fondo transparente
    $imagick->destroy();
}

// Información del certificado
$nombreFirmante = $certInfo['subject']['CN'] ?? 'Firmante desconocido';
$numeroSerie = $certInfo['subject']['serialNumber'] ?? 'Sin número de serie';
$razon = 'Certificado digital válido';
$ubicacion = $certInfo['subject']['L'] ?? 'Sin ubicación';

// Crear contenido para el QR
$contenidoQR = "Firmado por: $nombreFirmante\n";
$contenidoQR .= "Número de serie: $numeroSerie\n";
$contenidoQR .= "Fecha: " . date('Y-m-d H:i:s') . "\n";

// Generar el archivo QR
generarCodigoQR($contenidoQR, $rutaQR);

// Verificar si el QR se generó correctamente
if (file_exists($rutaQR)) {
    echo "El QR se generó correctamente en: $rutaQR\n";
} else {
    die("Error: No se pudo generar el QR.");
}

// Crear PDF con firma y QR
$pdf = new FPDI();
$rutaTemporalPDF = __DIR__ . '/Certificado.pdf';
$pageCount = $pdf->setSourceFile($rutaTemporalPDF);

for ($i = 1; $i <= $pageCount; $i++) {
    $tplIdx = $pdf->importPage($i);
    $pdf->AddPage();
    $pdf->useTemplate($tplIdx);
}

// Insertar logotipo
$rutaLogo = __DIR__ . '/logo_firma.png';
if (file_exists($rutaLogo)) {
    $pdf->Image($rutaLogo, 10, 10, 30);
}

// Alinear QR e información
$xQR = 20; // Posición X del QR
$yQR = 200; // Posición Y del QR

// Insertar código QR
if (file_exists($rutaQR)) {
    $pdf->Image($rutaQR, $xQR, $yQR, 40); // Tamaño del QR
}

// Información alineada junto al QR
$xTexto = $xQR + 50; // Texto a la derecha del QR
$yTexto = $yQR;

$pdf->SetTextColor(0, 0, 0);

// Configurar fuente más pequeña
$pdf->SetFont('Helvetica', '', 8); // Fuente 'Helvetica', tamaño 8

// Información del certificado más compacta
$pdf->SetXY($xTexto, $yTexto);
$pdf->Cell(0, 1, "Firmado por: $nombreFirmante", 0, 1, 'L'); // Espaciado reducido

$pdf->SetXY($xTexto, $yTexto + 5);
$pdf->Cell(0, 1, "Número de serie: $numeroSerie", 0, 1, 'L');

$pdf->SetXY($xTexto, $yTexto + 10);
$pdf->Cell(0, 1, "Fecha de firma: " . date('d/m/Y H:i:s'), 0, 1, 'L');

$pdf->SetXY($xTexto, $yTexto + 15);
$pdf->Cell(0, 1, "Razón: $razon", 0, 1, 'L');

// Firmar el PDF
$pdf->setSignature(
    'file://' . $publicCertPath,
    'file://' . $privateKeyPath,
    $nombreFirmante,
    '',
    2,
    [
        'Name' => $nombreFirmante,
        'Location' => $ubicacion,
        'Reason' => $razon,
        'ContactInfo' => 'goyaluis1@gmail.com'
    ]
);

// Guardar PDF firmado
$archivoFirmado = __DIR__ . '/Certificado_Firmado_QR.pdf';
$contador = 1;

// Verificar si el archivo ya existe
while (file_exists($archivoFirmado)) {
    $archivoFirmado = __DIR__ . '/Certificado_Firmado_QR_' . $contador . '.pdf';
    $contador++;
}

// Guardar el PDF firmado
$pdf->Output($archivoFirmado, 'F');
echo "PDF firmado correctamente en: $archivoFirmado";

// Limpieza
unlink($publicCertPath);
unlink($privateKeyPath);
unlink($rutaQR);

echo "PDF firmado correctamente en: $archivoFirmado";















/**
* C:\wamp64\www\tcpdf\index.php:85:
array (size=16)
'name' => string '/CN=LUIS ALBERTO GOYA ANCHUNDIA/serialNumber=170423141527/OU=ENTIDAD DE CERTIFICACION DE
INFORMACION/O=SECURITY DATA S.A. 2/C=EC' (length=128)
'subject' =>
array (size=5)
'CN' => string 'LUIS ALBERTO GOYA ANCHUNDIA' (length=27)
'serialNumber' => string '170423141527' (length=12)
'OU' => string 'ENTIDAD DE CERTIFICACION DE INFORMACION' (length=39)
'O' => string 'SECURITY DATA S.A. 2' (length=20)
'C' => string 'EC' (length=2)
'hash' => string '7d3baf98' (length=8)
'issuer' =>
array (size=4)
'C' => string 'EC' (length=2)
'O' => string 'SECURITY DATA S.A. 2' (length=20)
'OU' => string 'ENTIDAD DE CERTIFICACION DE INFORMACION' (length=39)
'CN' => string 'AUTORIDAD DE CERTIFICACION SUBCA-2 SECURITY DATA' (length=48)
'version' => int 2
'serialNumber' => string '623928840' (length=9)
'serialNumberHex' => string '25306608' (length=8)
'validFrom' => string '230417190551Z' (length=13)
'validTo' => string '250416190551Z' (length=13)
'validFrom_time_t' => int 1681758351
'validTo_time_t' => int 1744830351
'signatureTypeSN' => string 'RSA-SHA256' (length=10)
'signatureTypeLN' => string 'sha256WithRSAEncryption' (length=23)
'signatureTypeNID' => int 668
'purposes' =>
array (size=9)
1 =>
array (size=3)
0 => boolean true
1 => boolean false
2 => string 'sslclient' (length=9)
2 =>
array (size=3)
0 => boolean true
1 => boolean false
2 => string 'sslserver' (length=9)
3 =>
array (size=3)
0 => boolean true
1 => boolean false
2 => string 'nssslserver' (length=11)
4 =>
array (size=3)
0 => boolean true
1 => boolean false
2 => string 'smimesign' (length=9)
5 =>
array (size=3)
0 => boolean true
1 => boolean false
2 => string 'smimeencrypt' (length=12)
6 =>
array (size=3)
0 => boolean false
1 => boolean false
2 => string 'crlsign' (length=7)
7 =>
array (size=3)
0 => boolean true
1 => boolean true
2 => string 'any' (length=3)
8 =>
array (size=3)
0 => boolean true
1 => boolean false
2 => string 'ocsphelper' (length=10)
9 =>
array (size=3)
0 => boolean false
1 => boolean false
2 => string 'timestampsign' (length=13)
'extensions' =>
array (size=25)
'basicConstraints' => string 'CA:FALSE' (length=8)
'authorityKeyIdentifier' => string 'keyid:8C:BA:CA:11:57:78:25:80:1D:6B:0A:4B:55:BF:8D:AE:62:DD:BD:8F
' (length=66)
'authorityInfoAccess' => string 'OCSP - URI:http://ocspgw.securitydata.net.ec/ejbca/publicweb/status/ocsp
' (length=73)
'freshestCRL' => string '
Full Name:
URI:ldap://ldapsdca2.securitydata.net.ec/CN=AUTORIDAD DE CERTIFICACION SUBCA-2 SECURITY DATA,OU=ENTIDAD DE CERTIFICACION
DE INFORMACION,O=SECURITY DATA S.A. 2,C=EC?deltaRevocationList?base
' (length=203)
'subjectAltName' => string 'email:goyaluis1@gmail.com' (length=25)
'certificatePolicies' => string 'Policy: 1.3.6.1.4.1.37746.2.7
User Notice:
Explicit Text:
Policy: 1.3.6.1.4.1.37746.2.1
CPS: https://www.securitydata.net.ec/wp-content/downloads/Normativas/P_de_Certificados/Politicas de Certificado Persona
Natural.pdf
' (length=229)
'crlDistributionPoints' => string '
Full Name:
URI:http://ocspgw.securitydata.net.ec/ejbca/publicweb/status/ocsp
CRL Issuer:
DirName:CN = AUTORIDAD DE CERTIFICACION SUBCA-2 SECURITY DATA, OU = ENTIDAD DE CERTIFICACION DE INFORMACION, O =
SECURITY DATA S.A. 2, C = EC

Full Name:
URI:ldap://ldapsdca2.securitydata.net.ec/CN=AUTORIDAD DE CERTIFICACION SUBCA-2 SECURITY DATA,OU=ENTIDAD DE CERTIFICACION
DE INFORMACION,O=SECURITY DATA S.A. 2,C=EC?certificateRevocationList?base

Full Name:
URI:https://portal-operador2.securitydata.net.ec/ejbc'... (length=679)
'subjectKeyIdentifier' => string 'CD:4C:A0:B3:C3:ED:71:99:75:1C:29:86:6E:B9:92:0B:D1:10:6B:B9' (length=59)
'privateKeyUsagePeriod' => string 'Not Before: Apr 17 19:05:51 2023 GMT, Not After: Apr 16 19:05:51 2025 GMT'
(length=73)
'keyUsage' => string 'Digital Signature, Non Repudiation, Key Encipherment' (length=52)
'1.3.6.1.4.1.37746.3.1' => string '
0802481788' (length=12)
'1.3.6.1.4.1.37746.3.9' => string 'QUININDE' (length=10)
'1.3.6.1.4.1.37746.3.34' => string '.' (length=3)
'1.3.6.1.4.1.37746.3.7' => string ' AV PALCIEN S N Y NUEVA JERUSALEN' (length=34)
'1.3.6.1.4.1.37746.3.2' => string 'LUIS ALBERTO' (length=14)
'1.3.6.1.4.1.37746.3.32' => string '001002000386132' (length=17)
'1.3.6.1.4.1.37746.3.35' => string '.' (length=3)
'1.3.6.1.4.1.37746.3.33' => string 'PFX' (length=5)
'1.3.6.1.4.1.37746.3.12' => string 'ECUADOR' (length=9)
'1.3.6.1.4.1.37746.3.3' => string 'GOYA' (length=6)
'1.3.6.1.4.1.37746.3.30' => string '.' (length=3)
'1.3.6.1.4.1.37746.3.11' => string '
0802481788001' (length=15)
'1.3.6.1.4.1.37746.3.29' => string '.' (length=3)
'1.3.6.1.4.1.37746.3.4' => string ' ANCHUNDIA' (length=11)
'1.3.6.1.4.1.37746.3.8' => string '
0984769674' (length=12)
*/